package com.cultural.heritage.listener;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.constant.MqConstant;
import com.cultural.heritage.constant.OrderStatusConstant;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.snapshot.CouponSnapshot;
import com.cultural.heritage.model.entity.Order;
import com.cultural.heritage.model.entity.PendingServiceOrder;
import com.cultural.heritage.model.entity.UserCoupon;
import com.cultural.heritage.service.common.CommonService;
import com.cultural.heritage.service.good.UserCouponService;
import com.cultural.heritage.service.order.OrderService;
import com.cultural.heritage.service.order.PendingServiceOrderService;
import com.cultural.heritage.utils.MultiDelayMessage;
import jakarta.annotation.Resource;
import org.springframework.amqp.rabbit.annotation.Exchange;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.QueueBinding;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.stereotype.Component;

@Component
public class OrderStatusListener {


    @Resource
    private OrderService orderService;


    @Resource
    private RabbitTemplate rabbitTemplate;


    @Resource
    private UserCouponService userCouponService;


    @Resource
    private PendingServiceOrderService pendingServiceOrderService;


    @Resource
    private CommonService commonService;



    @RabbitListener(bindings = @QueueBinding(
            value = @Queue(MqConstant.DELAY_ORDER_QUEUE),
            exchange = @Exchange(name = MqConstant.DELAY_EXCHANGE, delayed = "true"),
            key = MqConstant.DELAY_ORDER_ROUTING_KEY
    ))
    public void listenDelayMessage(MultiDelayMessage<Long> msg) {
        System.out.println("\n\n\n\n\nOrderStatusListener.listenerDelayMessage msg-------------------------------->：" + msg);
        // 1.获取消息中的订单id
        Long orderId = msg.getData();
        // 2.查询订单，判断状态是否为待支付
        Order order = orderService.getById(orderId);
        // 订单不存在或者订单已经支付
        if (order == null || !order.getOrderStatus().equals(OrderStatusConstant.PENDING_PAYMENT)) {
            return ;
        }
        // 3.订单未支付，判断是否还有剩余延时时间
        if (msg.hasNextDelay()) {
            // 有延迟时间，需要重发延迟消息，先获取延迟时间的int值
            // 发送延时消息
            int delayValue = msg.removeNextDelay().intValue();
            rabbitTemplate.convertAndSend(MqConstant.DELAY_EXCHANGE,
                MqConstant.DELAY_ORDER_ROUTING_KEY, msg, message -> {
                    // 添加延迟消息属性
                    message.getMessageProperties().setDelay(delayValue);
                    return message;
            });
            return ;
        }
        // 没有剩余延时时间，说明订单超时未支付，需取消订单
        UpdateWrapper<Order> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", orderId).set("orderStatus", OrderStatusConstant.TRANSACTION_CLOSED);
        boolean update = orderService.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "订单状态更新失败");

        // 如果使用了优惠券，则退还优惠券
        CouponSnapshot couponSnapshot = order.getCouponSnapshot();
        if (couponSnapshot != null) {
            Long userCouponId = couponSnapshot.getId();
            UserCoupon userCoupon = userCouponService.getById(userCouponId);
            ThrowUtils.throwIf(userCoupon == null, ErrorCode.OPERATION_ERROR, "优惠券不存在");
            // 更新优惠券状态
            userCoupon.setIsUsed(0);
            boolean result = userCouponService.updateById(userCoupon);
            ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "优惠券状态更新失败");
        }

        boolean isGeneral = orderService.isGeneralGood(order.getOrderType());
        if (!isGeneral) {
            System.out.println("=========================================>" + order.getOrderNumber());
            UpdateWrapper<PendingServiceOrder> pendingServiceOrderUpdateWrapper = new UpdateWrapper<>();
            pendingServiceOrderUpdateWrapper.eq("orderNumber", order.getOrderNumber()).set("orderItemStatus", OrderStatusConstant.TRANSACTION_CLOSED);
            boolean updateBatch = pendingServiceOrderService.update(pendingServiceOrderUpdateWrapper);
            ThrowUtils.throwIf(!updateBatch, ErrorCode.OPERATION_ERROR, "订单状态批量更新失败");
        }
    }


}
