package com.cultural.heritage.service.order;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.cultural.heritage.model.dto.cart.CartOrderAddRequest;
import com.cultural.heritage.model.dto.cartService.CartExperienceOrderAddRequest;
import com.cultural.heritage.model.dto.order.OrderAddRequest;
import com.cultural.heritage.model.dto.order.OrderQueryRequest;
import com.cultural.heritage.model.dto.order.capital.GeneralGoodSingleBuyAddRequest;
import com.cultural.heritage.model.dto.order.capital.OrderMainInfoAddRequest;
import com.cultural.heritage.model.entity.Order;

import java.math.BigDecimal;
import java.util.List;

public interface OrderService extends IService<Order> {


    /**
     * 获取查询条件
     */
    QueryWrapper<Order> getQueryWrapper(OrderQueryRequest orderQueryRequest);


    /**
     * 封装订单详细信息请求体
     */
    OrderAddRequest initOrderMainInfo(OrderMainInfoAddRequest orderMainInfoAddRequest);


    /**
     * 校验订单
     */
    void validOrder(Order order);



    /**
     * 创建通用订单(常规类和服务类商品)
     */
    Long createCommonOrder(OrderMainInfoAddRequest orderMainInfoAddRequest, Long userId, boolean isCartOrder, List<Long> cartIds, boolean isGeneral);


    /**
     * 校验单独购买的商品数据是否准确
     */
    void validSingleGoodOrder(GeneralGoodSingleBuyAddRequest generalGoodSingleBuyAddRequest, boolean isGeneral);


    /**
     * 计算单个商品购买的总金额
     */
    BigDecimal calculateTotalAmount(OrderMainInfoAddRequest orderMainInfoAddRequest);


    /**
     * 向消息队列中发送订单创建的消息
     */
    void sendCreateOrderMessage(Long orderId);


    /**
     * 判断是否是常规类商品
     */
    boolean isGeneralGood(String orderType);


    /**
     * 常规类商品购物车封装订单主要信息请求体
     */
    OrderMainInfoAddRequest packagedOrderMainInfoAddRequest(CartOrderAddRequest cartOrderAddRequest);


    /**
     * 服务类商品购物车封装订单主要信息请求体
     */
    OrderMainInfoAddRequest packagedOrderMainInfoAddRequest(CartExperienceOrderAddRequest cartExperienceOrderAddRequest);

}
